#include <syslog.h>
#include <sys/select.h>
#include <errno.h>

#include "safe_write.h"


ssize_t safe_write(int fd, const void *buf, size_t count) {
  int res;
  fd_set rdfs, wrfs;
  struct timeval tv = {0, 0};
  char read_buf[_SAFE_WRITE_READ_BUF_SIZE+1];

  FD_ZERO(&rdfs);
  FD_ZERO(&wrfs);
  FD_SET(fd, &rdfs);
  FD_SET(fd, &wrfs);
  
  while (1) {
    res = select(fd+1, &rdfs, &wrfs, NULL, &tv);
    if (-1 == res) {
      syslog(LOG_DEBUG, "_safe_write: error in select: %d, %s", errno, strerror(errno));
      return -1;
    } else {
      if (FD_ISSET(fd, &rdfs)) {
	/* set to read: something is wrong */
	syslog(LOG_DEBUG, "_safe_write, set to read");
	res = read(fd, read_buf, _SAFE_WRITE_READ_BUF_SIZE);
	syslog(LOG_DEBUG, "_safe_write, read res: %d", res);
	if (0 == res) {
	  syslog(LOG_DEBUG, "_safe_write, read returns 0, socket probably closed");
	  return -1;
	}
      } else if (FD_ISSET(fd, &wrfs)) {
	/* set to write */
	syslog(LOG_DEBUG, "_safe_write, set to write");
	res = write(fd, buf, count);
	return res;
      } else {
	syslog(LOG_DEBUG, "_safe_write: whether read nor write ready, strange");
	return -1;
      }
    }
  }
}

